﻿using System;
using System.IO;

using iTextSharp.text.pdf;
using iTextSharp.text.pdf.parser;
using System.Text;
using System.Runtime.InteropServices;
using System.Collections.Generic;

namespace PdfImage.Helpers
{
    /// <summary>
    /// Helper lass to dump all images from a PDF into separate files
    /// </summary>
    public class ImageExtractor : IRenderListener
    {
        int _currentPage = 1;
        int _imageCount = 0;
        int _i = 0;
        string _outputFilePrefix;
        string _outputFolder;
        bool _overwriteExistingFiles;

        public ImageExtractor(string outputFilePrefix, string outputFolder, bool overwriteExistingFiles)
        {
            _outputFilePrefix = outputFilePrefix;
            _outputFolder = outputFolder;
            _overwriteExistingFiles = overwriteExistingFiles;
        }

        /// <summary>
        /// Extract all images from a PDF file
        /// </summary>
        /// <param name="pdfPath">Full path and file name of PDF file</param>
        /// <param name="outputFilePrefix">Basic name of exported files. If null then uses same name as PDF file.</param>
        /// <param name="outputFolder">Where to save images. If null or empty then uses same folder as PDF file.</param>
        /// <param name="overwriteExistingFiles">True to overwrite existing image files, false to skip past them</param>
        /// <returns>Count of number of images extracted.</returns>
        public static int ExtractImagesFromFile(string pdfPath, string outputFilePrefix, string outputFolder, bool overwriteExistingFiles)
        {
            // Handle setting of any default values
            outputFilePrefix = outputFilePrefix ?? System.IO.Path.GetFileNameWithoutExtension(pdfPath);
            outputFolder = String.IsNullOrEmpty(outputFolder) ? System.IO.Path.GetDirectoryName(pdfPath) : outputFolder;

            var instance = new ImageExtractor(outputFilePrefix, outputFolder, overwriteExistingFiles);

            using (var pdfReader = new PdfReader(pdfPath))
            {
                if (pdfReader.IsEncrypted())
                    throw new ApplicationException(pdfPath + " is encrypted.");

                var pdfParser = new PdfReaderContentParser(pdfReader);

                while (instance._currentPage <= pdfReader.NumberOfPages)
                {


                    #region Crea folder per pagina
                    DirectoryInfo outPutDir = new DirectoryInfo(outputFolder + "\\" + instance._currentPage);
                    if (!outPutDir.Exists)
                    {
                        outPutDir.Create();
                        instance._outputFolder = outPutDir.FullName;
                    }
                    #endregion Crea folder per pagina

                    #region Esporta Testo
                    StringBuilder text = new StringBuilder();
                    text.Append(PdfTextExtractor.GetTextFromPage(pdfReader, instance._currentPage));
                    pdfParser.ProcessContent(instance._currentPage, instance);

                    
                    using (FileStream fs = File.Create(instance._outputFolder + "\\" + instance._currentPage.ToString() + ".txt"))
                    {
                        byte[] info = new UTF8Encoding(true).GetBytes(text.ToString());
                        // Add some information to the file.
                        fs.Write(info, 0, info.Length);
                    }
                    #endregion Esporta Testo


                    string[] lines = text.ToString().Split(Environment.NewLine.ToCharArray());
                    int countLines = lines.Length;


                    NormalizeFolder(instance._outputFolder);

                    #region Prima Pagina
                    if (instance._currentPage == 1)
                    { 
                        for (int i=1;i< countLines; ++i)
                        {
                            //Rinomina File
                            string line = lines[i];
                            string newFileName = line.Substring(0, 7);
                            newFileName.Trim();
                            string fileSearchJpg = instance._outputFolder + "\\" + "img._" + instance._currentPage.ToString() + "_" + (i - 1).ToString() + ".jpg";
                            
                            FileInfo fileImg = new FileInfo(fileSearchJpg);
                            fileImg.MoveTo(instance._outputFolder + "\\" + newFileName + ".jpg");
                            
                        }
                    }
                    #endregion Prima Pagina
                    #region Altre Pagina
                    else
                    {
                        for (int i = 0;  i< countLines; ++i)
                        {
                            if (i == 14)
                                System.Diagnostics.Debug.WriteLine("Stop");
                            //Rinomina File
                            string line = lines[i];
                            string newFileName = line.Substring(0, 7);
                            newFileName.Trim();
                            string fileSearchJpg = instance._outputFolder + "\\" + "img._" + instance._currentPage.ToString() + "_" + i.ToString() + ".jpg";

                            FileInfo fileImg = new FileInfo(fileSearchJpg);                           
                            fileImg.MoveTo(instance._outputFolder + "\\" + newFileName + ".jpg");
                            
                        }
                    }
                    #endregion Altre Pagina
                    
                    instance._currentPage++;
                    instance._imageCount = 0;


                }
            }

            return instance._imageCount;
        }

        #region Implementation of IRenderListener

        public void BeginTextBlock()
        {
        
        }
        public void EndTextBlock()
        {
        
        }
        public void RenderText(TextRenderInfo renderInfo)
        {

        }

        public void RenderImage(ImageRenderInfo renderInfo)
        {
            
            var imageObject = renderInfo.GetImage();
            
            var imageFileName = String.Format("{0}_{1}_{2}.{3}", _outputFilePrefix, _currentPage, _imageCount, imageObject.GetFileType());
            var imagePath = System.IO.Path.Combine(_outputFolder, imageFileName);

            if (_overwriteExistingFiles || !File.Exists(imagePath))
            {
                var imageRawBytes = imageObject.GetImageAsBytes();

                File.WriteAllBytes(imagePath, imageRawBytes);

            }

            // Subtle: Always increment even if file is not written. This ensures consistency should only some
            //   of a PDF file's images actually exist.
            _imageCount++;

            }

        #endregion // Implementation of IRenderListener

        public static void NormalizeFolder(string path)
        {
            try
            { 

                string[] imgFiles = Directory.GetFiles(path, "img*");


                int count = imgFiles.Length;

                int delCount = 0;

                for (int i = 0; i < count; ++i)
                {
                    FileInfo imgFile = new FileInfo(imgFiles[i]);
                    if (imgFile.Length < 200)
                    {
                        imgFile.Delete();
                        delCount++;
                    }
                    else
                    {
                        if (imgFile.Extension == ".png")
                        {
                            string tmpName = imgFile.FullName;
                            tmpName = tmpName.Replace("png", "jpg");
                            imgFile.MoveTo(tmpName);
                        }
                    }
                }

                imgFiles = Directory.GetFiles(path, "img*");

                Array.Sort(imgFiles, new MyComparer());

                count = imgFiles.Length;

                for (int i = 0; i < count; ++i)
                {
                    FileInfo imgFile = new FileInfo(imgFiles[i]);
                    string oldFileName = imgFile.Name;

                    String[] spearator = { "._", "_", ".", "." };
                    Int32 countSep = 4;
                    String[] strlist = oldFileName.Split(spearator, countSep, StringSplitOptions.RemoveEmptyEntries);

                    string newFileName = path + "\\" + strlist[0] + "._" + strlist[1] + "_" + (int.Parse(strlist[2]) - delCount).ToString() + ".jpg";


                    imgFile.MoveTo(newFileName);

                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            
        }

        public static string ExtractTextFromPdf(string path)
        {
            using (PdfReader reader = new PdfReader(path))
            {
                StringBuilder text = new StringBuilder();

                for (int i = 1; i <= reader.NumberOfPages; i++)
                {
                    text.Append(PdfTextExtractor.GetTextFromPage(reader, i));
                }

                return text.ToString();
            }
        }
    }

    public class MyComparer : IComparer<string>
    {

        [DllImport("shlwapi.dll", CharSet = CharSet.Unicode, ExactSpelling = true)]
        static extern int StrCmpLogicalW(String x, String y);

        public int Compare(string x, string y)
        {
            return StrCmpLogicalW(x, y);
        }

    }
}
